import os
import sys
import socket
import stat
import errno
#from debug import *
import json


def parse_req(req):
  return json.loads(req)


def format_req(service, method, *args, **kwargs):
  return json.dumps([service, method, args, kwargs])


def parse_resp(resp):
  return json.loads(resp)


def format_resp(resp):
  return json.dumps(resp)


def buffered_readlines(sock):
  buf = ''
  while True:
    while '\n' in buf:
      (line, nl, buf) = buf.partition('\n')
      yield line
    try:
      newdata = sock.recv(4096)
      if newdata == '':
        break
      buf += newdata
    except IOError, e:
      if e.errno == errno.ECONNRESET:
        break


class RpcServer(object):

  def __init__(self, services):
    self.services = {}
    for service in services:
      print ('[INFO] Providing service %s' % service)
      module, clazz = service.rsplit('.', 1)
      print ('[INFO] Module is %s, class/func is %s' % (module, clazz))
      mod = __import__(module, fromlist=[clazz])
      obj = getattr(mod, clazz)
      is_fn = False
      if hasattr(obj, '__call__'):
        is_fn = True
      else:
        obj = obj()
      self.services[clazz] = (is_fn, obj)

  def run_sock(self, sock):
    lines = buffered_readlines(sock)
    for req in lines:
      service, method, args, kwargs = parse_req(req)
      print '[INFO] [Service Provider] Received request to call %s, method %s with args %s kwargs %s' % (
          service, method, args, kwargs)
      if service not in self.services:
        print '[INFO] Oops, I don\'t provide service %s' % service
        return
      service_data = self.services[service]
      ret = ''
      if service_data[0]:
        ret = service_data[1](*args, **kwargs)
      else:
        ret = getattr(service_data[1], method)(*args, **kwargs)
      sock.sendall(format_resp(ret) + '\n')

  def run_sockpath_fork(self, sockpath):
    print '[INFO] Using sockpath %s' % sockpath
    open(sockpath, 'w')
    os.unlink(sockpath)
    if os.path.exists(sockpath):
      s = os.stat(sockpath)
      if not stat.S_ISSOCK(s.st_mode):
        raise Exception('%s exists and is not a socket' % sockpath)
      os.unlink(sockpath)

    server = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    server.bind(sockpath)

    # Allow anyone to connect.
    # For access control, use directory permissions.
    os.chmod(sockpath, 0777)

    server.listen(5)
    while True:
      conn, addr = server.accept()
      pid = os.fork()
      if pid == 0:
        # fork again to avoid zombies
        if os.fork() <= 0:
          self.run_sock(conn)
          sys.exit(0)
        else:
          sys.exit(0)
      conn.close()
      os.waitpid(pid, 0)


class RpcClient(object):

  def __init__(self, sock):
    self.sock = sock
    self.lines = buffered_readlines(sock)

  def call(self, service, method, *args, **kwargs):
    self.sock.sendall(format_req(service, method, *args, **kwargs) + '\n')
    return parse_resp(self.lines.next())

  def close(self):
    self.sock.close()

    ## __enter__ and __exit__ make it possible to use RpcClient()
    ## in a "with" statement, so that it's automatically closed.
  def __enter__(self):
    return self

  def __exit__(self, *args):
    self.close()


def client_connect(pathname):
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    sock.connect(pathname)
    return RpcClient(sock)
